@@ -2,8 +2,8 @@
 
 from django import template
 register = template.Library()
-from djaloha.templatetags.djaloha_utils import DjalohaEditNode
-from coop_cms.models import PieceOfHtml, BaseArticle
+from djaloha.templatetags.djaloha_utils import DjalohaEditNode, DjalohaMultipleEditNode
+from coop_cms.models import PieceOfHtml, BaseArticle, Fragment, FragmentType, FragmentFilter
 from django.utils.translation import ugettext_lazy as _
 from django.core.context_processors import csrf
 from django.utils.safestring import mark_safe
@@ -16,7 +16,7 @@ logger = logging.getLogger("coop_cms")
 ################################################################################
 class PieceOfHtmlEditNode(DjalohaEditNode):
     def render(self, context):
-        if context.get('form', None):
+        if context.get('form', None) or context.get('formset', None):
             context.dicts[0]['djaloha_edit'] = True
         #context.dicts[0]['can_edit_template'] = True
         return super(PieceOfHtmlEditNode, self).render(context)
@@ -26,6 +26,49 @@ def coop_piece_of_html(parser, token):
     div_id = token.split_contents()[1]
     return PieceOfHtmlEditNode(PieceOfHtml, {'div_id': div_id}, 'content')
 
+################################################################################
+
+class FragmentEditNode(DjalohaMultipleEditNode):
+    
+    def _get_objects(self, lookup):
+        self.fragment_type, _x = FragmentType.objects.get_or_create(name=lookup['name'])
+        qs = Fragment.objects.filter(type=self.fragment_type)
+        if lookup.has_key('extra_id'):
+            self.fragment_filter, _x = FragmentFilter.objects.get_or_create(extra_id=lookup['extra_id'])
+            qs = qs.filter(filter=self.fragment_filter)
+        return qs
+    
+    def _get_object_lookup(self, obj):
+        return {"id": obj.id}
+
+    def __init__(self, lookup):
+        super(FragmentEditNode, self).__init__(Fragment, lookup, 'content')
+        self.fragment_filter = None
+    
+    def _pre_object_render(self, obj):
+        return u'<div class="coop-fragment {0}" rel="{1}">'.format(obj.css_class, obj.id)
+    
+    def _post_object_render(self, obj):
+        return u'</div>'
+    
+    def render(self, context):
+        if context.get('form', None) or context.get('formset', None):
+            context.dicts[0]['djaloha_edit'] = True
+        #context.dicts[0]['can_edit_template'] = True
+        html = super(FragmentEditNode, self).render(context)
+        filter_id = self.fragment_filter.id if self.fragment_filter else 0
+        pre_html = u'<div style="display: none" class="coop-fragment-type" rel="{0}" data-filter="{2}">{1}</div>'.format(
+            self.fragment_type.id, self.fragment_type.name, filter_id)
+        return pre_html+html
+
+@register.tag
+def coop_fragments(parser, token):
+    args = token.split_contents()
+    lookup = {'name': args[1]}
+    if len(args) > 2:
+        lookup["extra_id"] = args[2]
+    return FragmentEditNode(lookup)
+
 ################################################################################
 class ArticleSummaryEditNode(DjalohaEditNode):
     def render(self, context):
@@ -63,16 +106,16 @@ class CmsFormMediaNode(template.Node):
 
     def render(self, context):
         form = context.get('form', None)
-        if form:
+        formset = context.get('formset', None)
+        if form or formset:
             t = template.Template("{{form.media}}")
-            html = t.render(template.Context({'form': form}))
+            html = t.render(template.Context({'form': form or formset}))
             #django 1.5 fix : " are escaped as &quot; and cause script tag 
             #for aloha to be broken
             return html.replace("&quot;", '"') 
         else:
             return ""
 
-
 @register.tag
 def cms_form_media(parser, token):
     return CmsFormMediaNode()
@@ -102,7 +145,7 @@ class IfCmsEditionNode(template.Node):
             yield node
 
     def _check_condition(self, context):
-        return context.get('form', None)
+        return context.get('form', None) or context.get('formset', None)
 
     def render(self, context):
         if self._check_condition(context):
@@ -142,7 +185,10 @@ class SafeWrapper:
 
     def __getattr__(self, field):
         value = getattr(self._wrapped, field)
-        if field=='logo':
+        
+        if hasattr(value, 'pk'):
+            return value
+        elif field=='logo':
             src = getattr(self._wrapped, 'logo_thumbnail')(False, self._logo_size)
             if src:
                 try:
@@ -160,9 +206,9 @@ class SafeWrapper:
 
 class FormWrapper:
 
-    def __init__(self, form, the_object, logo_size=None):
+    def __init__(self, form, obj, logo_size=None):
         self._form = form
-        self._obj = the_object
+        self._obj = obj
         if logo_size:
             self._form.set_logo_size(logo_size)
 
@@ -187,9 +233,13 @@ class CmsEditNode(template.Node):
             yield node
 
     def render(self, context):
-        form = context.get('form', None)
         request = context.get('request')
-        the_object = context.get(self.var_name)
+        
+        form = context.get('form', None)
+        obj = context.get(self.var_name, None) if self.var_name else None
+        
+        formset = context.get('formset', None)
+        objects = context.get('objects', None)
 
         #the context used for rendering the templatetag content
         inner_context = {}
@@ -197,26 +247,37 @@ class CmsEditNode(template.Node):
             inner_context.update(x)
 
         #the context used for rendering the whole page
-        self.post_url = the_object.get_edit_url()
+        self.post_url = obj.get_edit_url() if obj else context.get('coop_cms_edit_url')
         outer_context = {'post_url': self.post_url}
 
-        inner_context[self.var_name] = the_object
+        if self.var_name:
+            inner_context[self.var_name] = obj
+        if formset:
+            inner_context['formset'] = formset
+        if objects:
+            inner_context['objects'] = objects
 
         safe_context = inner_context.copy()
-        inner_context[self.var_name] = the_object
+        #inner_context[self.var_name] = obj
         inner_value = u""
 
-        if form:
+        if form or formset:
             t = template.Template(CMS_FORM_TEMPLATE)
-            safe_context[self.var_name] = FormWrapper(form, the_object, logo_size=self._logo_size)
+            if form:
+                safe_context[self.var_name] = FormWrapper(form, obj, logo_size=self._logo_size)
+            else:
+                safe_context['objects'] = [FormWrapper(f, o, logo_size=self._logo_size) for (f, o) in zip(formset, objects)]
             outer_context.update(csrf(request))
             #outer_context['inner'] = self.nodelist_content.render(template.Context(inner_context))
         else:
             t = template.Template("{{inner|safe}}")
-            safe_context[self.var_name] = SafeWrapper(the_object, logo_size=self._logo_size)
+            if obj:
+                safe_context[self.var_name] = SafeWrapper(obj, logo_size=self._logo_size)
+            else:
+                safe_context['objects'] = [SafeWrapper(o, logo_size=self._logo_size) for o in objects]
                 
         managed_node_types = [
-            template.TextNode, template.defaulttags.IfNode,
+            template.TextNode, template.defaulttags.IfNode, template.defaulttags.ForNode,
             IfCmsEditionNode, IfNotCmsEditionNode,
         ]
                 
@@ -225,7 +286,9 @@ class CmsEditNode(template.Node):
             if any([isinstance(node, node_type) for  node_type in managed_node_types]):
                 c = node.render(template.Context(safe_context))
             elif isinstance(node, template.loader_tags.BlockNode):
-                c = node.render(context)
+                sf = template.Context(safe_context)
+                sf.render_context['block_context'] = context.render_context.get('block_context', None)
+                c = node.render(sf)
             elif isinstance(node, template.VariableNode):
                 if node.filter_expression.filters:
                     c = node.render(context)
@@ -234,14 +297,14 @@ class CmsEditNode(template.Node):
             else:
                 c = node.render(template.Context(inner_context))
             inner_value += c
-        outer_context['inner'] = mark_safe(inner_value) if form else inner_value
+        outer_context['inner'] = mark_safe(inner_value) if (form or formset) else inner_value
         return t.render(template.Context(outer_context))
 
 @register.tag
 def cms_edit(parser, token):
     args = token.split_contents()[1:]
     data = {}
-    var_name = args[0]
+    var_name = args[0] if len(args) else ''
     for arg in args[1:]:
         k, v = arg.split('=')
         data[k] = v
